/********************************************************************************
 * * Archivo: mpi_prime.c
 * * DESCRIPCION: Calcula el nmero primo mayor de un nmero dado(LIMIT) e indica
 * * cuantos nmeros primos existen en este rango.
 * * EVENTO: Congreso Nacional de Software Libre 2004(CONSOL) www.consol.org.mx/2004/
 * * PONENTES:
 * *    Castillo Castillo Jos - www.josecc.net
 * *    Ricardo Benjamn Olvera Acosta
 * *    Jeanette Moreno Snchez.
 * * AUTHOR: Blaise Barney 11/25/95 - adapted from version contributed by 
 * *    Richard Ng &  Wong Sze Cheong during MHPCC Singapore Workshop (8/22/95).
 * * URL: http://www.columbia.edu/itc/applied/e6901/
 * * NOTA: Slo se utiliza este cdigo como ejemplo de demostracin. Se ha modificado
 * *    algunas lneas con fines didacticos, puede encontrar el cdigo original en
 * *    el sitio antes referido.
 * ******************************************************************************/

#include "mpi.h"
#include <stdio.h>
#include <math.h>

#define LIMIT     2500000*100*2     /* Incremente para calcular un nmero primo mayor*/
#define FIRST     0           /* Rank of first task */

int isprime(int n) {
  int i,squareroot;
  if (n>10) {
    squareroot = (int) sqrt(n);
    for (i=3; i<=squareroot; i=i+2)
      if ((n%i)==0)
	return 0;
    return 1;
  }
  /* Assume first four primes are counted elsewhere. Forget everything else */
  else
    return 0;
}


 main(argc,argv)
     int argc;
     char *argv[]; {

       int   ntasks,           /* total number of tasks in partitiion */
	 rank,                 /* task identifier */
	 n,                    /* loop variable */
	 pc,                   /* prime counter */
	 pcsum,                /* number of primes found by all tasks */
	 foundone,             /* most recent prime found */
	 maxprime,             /* largest prime found */
	 mystart,              /* where to start calculating */
	 stride;               /* calculate every nth number */

       double start_time,end_time;
								        
       MPI_Init(&argc,&argv);
       MPI_Comm_rank(MPI_COMM_WORLD,&rank);
       MPI_Comm_size(MPI_COMM_WORLD,&ntasks);
       if (((ntasks%2) !=0) || ((LIMIT%ntasks) !=0)) {
	 printf("Programa terminado - este ejercicio requiere ejecutarse con 2, 4, 8, 16, 32, 64, etc. nmero de tareas(procesos)\n");
	 MPI_Finalize();
	printf("\n.---.\n");
       }

       start_time = MPI_Wtime();   /* Initialize start time */
       mystart = (rank*2)+1;       /* Find my starting point - must be odd number */
       stride = ntasks*2;          /* Determine stride, skipping even numbers */
       pc=0;                       /* Initialize prime counter */
       foundone = 0;               /* Initialize */

       /******************** task with rank 0 does this part ********************/
       if (rank == FIRST) {
	 printf("Usando %d tareas(procesos) para buscar en %d nmeros\n",ntasks,LIMIT);
	 pc = 4;                  /* Assume first four primes are counted here */
	 for (n=mystart; n<=LIMIT; n=n+stride) {
	   if (isprime(n)) {
	     pc++;
	     foundone = n;
	               /* printf("%d\n",foundone);*/
	   }
	 }
	 MPI_Reduce(&pc,&pcsum,1,MPI_INT,MPI_SUM,FIRST,MPI_COMM_WORLD);
	 MPI_Reduce(&foundone,&maxprime,1,MPI_INT,MPI_MAX,FIRST,MPI_COMM_WORLD);
	 end_time=MPI_Wtime();
	 printf("Terminado. El nmero primo mayor encontrado: %d. Total de nmeros primos: %d\n",maxprime,pcsum);
	 printf("Tiempo transcurrido: %.2lf segundos, %f minutos\n",end_time-start_time,(float)((end_time-start_time)/60.0));
	exit(0);
       }


       /******************** all other tasks do this part ***********************/
       if (rank > FIRST) {
	 for (n=mystart; n<=LIMIT; n=n+stride) {
	   if (isprime(n)) {
	     pc++;
	     foundone = n;
	      /*          printf("%d\n",foundone);*/
	   }
	 }
	 MPI_Reduce(&pc,&pcsum,1,MPI_INT,MPI_SUM,FIRST,MPI_COMM_WORLD);
	 MPI_Reduce(&foundone,&maxprime,1,MPI_INT,MPI_MAX,FIRST,MPI_COMM_WORLD);
       }

       MPI_Finalize();
	printf("\nTerminando ...\n");
     }

