#include <qregexp.h>
#include <qstring.h>

#include "utility.h"
#include "image.h"

// --------------------------------------------------------------------------------
bool imageBuilder::buildImage(int w,int h,QValueList<JOB *> &jobsList)
{
  m_img.create(w,h,32);
  
  for(QValueList<JOB *>::iterator i=jobsList.begin();i!=jobsList.end();i++)
  { 
    bool err;
    
    QImage tmp = buildJobImage(err,*i);
    
    if(!err)
    {
      for(int x=0;x<tmp.width();x++)
        for(int y=0;y<tmp.height();y++)
        { 
          QRgb rgb = tmp.pixel(x,y);
          
          m_img.setPixel(x,y+(*i)->getStart(),rgb);
        }
      
    }  
  }
  
  return(true);
}

// --------------------------------------------------------------------------------
QImage imageBuilder::buildJobImage(bool &err,JOB *job)
{
  QImage tmp;
  
  err = false;
  
  QFile In(job->getOutputFile());
  if(!In.open(IO_ReadOnly))
  { 
    err = true;
    return(tmp);
  }
  
  QTextStream stream(&In);

  QString H1;
  QString W,H;
  QString DEPTH;

  stream >> H1;
  stream >> W;
  stream >> H;
  stream >> DEPTH;

  if(INT(DEPTH)!=255)
  { 
    err = true;
    return(tmp);
  }
  
  int LEN = INT(W)*INT(H)*3;
  
  QString     outData;
  QTextStream Out(&outData,IO_WriteOnly);

  Out << H1 << "\n";
  Out << W << "\n";
  Out << STRING(job->getEnd()-job->getStart()+1) << "\n";
  Out << DEPTH << "\n";

  int SIZE = outData.length() + LEN;

  char *buffer = new char [SIZE];

  memcpy(buffer,(const char *) outData,outData.length());

  In.readBlock(buffer+outData.length(),LEN);

  tmp.loadFromData((unsigned char *) buffer,SIZE,"PPM"); 

  delete [] buffer;
  
  return(tmp);
}

// --------------------------------------------------------------------------------
QValueList<QString> imageBuilder::getSupportedOutputFormats(void)
{
  QImage              img;
  QValueList<QString> ret;
  
  QStringList           list = img.outputFormatList();
  QStringList::Iterator i = list.begin();
  
  while(i!=list.end())
  {
    ret.push_back((*i).lower());
    ++i;
  }
  
  return(ret);
}

// --------------------------------------------------------------------------------
bool imageBuilder::checkOutputFormat(const QString &outputFormat,QString &format)
{
  QImage       img;
  
  QStringList list = img.outputFormatList();
  QStringList::Iterator i = list.begin();
  
  while(i!=list.end())
  {
    QRegExp re(QString(".") + (*i).lower() + QString("$"));
    
    if(!re.isValid()) CERR("Warning : invalid ragexp expression !\n");
    
    if(re.search(outputFormat)!=-1)
    {
      format = (const char *) (*i);
      return(true);
    }
    
    ++i;
  }
  
  return(false);
}

// --------------------------------------------------------------------------------
bool imageBuilder::saveImage(const QString &filename,const QString &type)
{
  if(m_img.isNull()) return(false);

  return(m_img.save(filename,type));
}

// --------------------------------------------------------------------------------
bool imageBuilder::buildImage(int w,int h,JOB *job)
{
  QString str = job->getRealOutputName();
  
  CERR((QString("Reading : ") + str + "\n"));
  if(!m_img.load(str))
  { 
    CERR("Warning : cannot create frame image !\n");
    CERR((QString("image : ") + job->getOutputFile() + QString("\n")));
    return(false);
  } 
  
  if(m_img.width()!=w || m_img.height()!=h)
  { 
    CERR("Warning : cannot create frame image (size)!\n");
    return(false);
  } 
  
  return(true);
}
