#ifndef _JOB_H
#define _JOB_H

#include <qstring.h>
#include <qprocess.h>
#include <qlistview.h>

#include "utility.h"



class JOB : QProcess
{
  Q_OBJECT

  private:

  QString m_cmdString;
  QString m_attrString;
  QString m_outputDir;
  QString m_inputFile;
  QString m_outputFile;
  int     m_pid;
  int     m_width;
  int     m_height;
  int     m_start,m_end;
  int     m_task;
  int     m_job;
  enum    {QUEUED,RUNNING,FINISHED,PAUSED} m_status;
  QString m_stdOut;
  QString m_stdErr;
  int     m_exitStatus;
  int     m_rendering_p;
  bool    m_selectedFlag;
  int     m_frameNumber;
  int     m_frameTotal;

  QListViewItem *m_item;
  


  public:

  JOB(QString &error,int task,int job,const QString &cmdString,const QString &inputFile,const QString &attrString,const QString &outputDir,int width,int height,int start,int end);
  ~JOB(void);

  bool run(void);
  void stop(void);
  void pause_job(void);
  void continue_job(void);


  // --------------------------------------------------------------------------------
  // GET
  // --------------------------------------------------------------------------------
  int             getJobNumber(void)    const  {return(m_job);}
  int             getStart(void)        const  {return(m_start);}
  int             getEnd(void)          const  {return(m_end);}
  const QString & getTempFile(void)     const  {return(m_outputFile);}
  const QString & getStdOut(void)       const  {return(m_stdOut);}
  const QString & getStdErr(void)       const  {return(m_stdErr);}
  const QString & getOutputFile(void)   const  {return(m_outputFile);}
  const QString   getJobStatusStr(void) const;
  
  int             getW(void)            const  {return(m_width);}
  int             getH(void)            const  {return(m_height);}
  int             getExitStatus(void)   const  {return(m_exitStatus);}
  int             getPID(void)                 {return(processIdentifier());}
  bool            getSelectionFlag(void) const {return(m_selectedFlag);}

  int             getFrameNumber(void)   const {return(m_frameNumber);}
  int             getFrameTotal(void)    const {return(m_frameTotal);}
  
  const QString   getPerc(void)         const {return(STRING(m_rendering_p)+" %");}
  
  QString         getRealOutputName(void) const;
      
  // --------------------------------------------------------------------------------
  // SET
  // --------------------------------------------------------------------------------

  void setItem(QListViewItem *i)   {m_item=i;}
  void setSelectionFlag(bool f)    {m_selectedFlag=f;}
  void setFrameNumber(int n,int t) {m_frameNumber=n;m_frameTotal=t;}
    

  // --------------------------------------------------------------------------------
  // SLOTS
  // --------------------------------------------------------------------------------  
  
  private slots:
  
  void exitCallback(void);
  void stdOutCallback(void);
  void stdErrCallback(void);
};

#endif

