#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>

#include "utility.h"
#include "openm.h"


// --------------------------------------------------------------------------------
openMosixWrapper::openMosixWrapper(void)
{
  m_nodeId        = getId();
  m_autoMigration = getAM();
  m_info          = getInfo();
  getNodes();
  getNodesData();
  
  if(m_nodeId>0) m_isOk = true;
            else m_isOk = false;
}

// --------------------------------------------------------------------------------
int openMosixWrapper::getId(void)
{
  QFile In("/proc/hpc/admin/mospe");
  
  if(!In.open(IO_ReadOnly)) return(-1);
  
  QTextStream stream(&In);

  QString nodeStr;
  
  stream >> nodeStr;
  
  return(INT(nodeStr));
}

// --------------------------------------------------------------------------------
bool openMosixWrapper::getAM(void)
{
  QFile In("/proc/hpc/admin/stay");
  
  if(!In.open(IO_ReadOnly)) return(false);
  
  QTextStream stream(&In);

  QString stayStr;
  
  stream >> stayStr;
  
  return(INT(stayStr)==0 ? true : false);
}

// --------------------------------------------------------------------------------
QString openMosixWrapper::getInfo(void)
{
  QFile In("/proc/hpc/admin/version");
  
  if(!In.open(IO_ReadOnly)) return("Error !");
  
  QString infoStr;  

  while(!In.atEnd())
  { 
    char a;
    
    a = In.getch();
    infoStr += a;
  } 
  
  return(infoStr);
}

// --------------------------------------------------------------------------------
void openMosixWrapper::getNodes(void)
{
  m_nodes.clear();
  
  QDir d("/proc/hpc/nodes/");

  if(!d.exists()) return;

  d.setSorting(QDir::Name | QDir::Reversed);

  const QFileInfoList *list = d.entryInfoList();
  
  QFileInfoListIterator it(*list);
  QFileInfo *fi;

  while ((fi = it.current()) != 0)
  {
    if(fi->fileName()!="." && fi->fileName()!="..")
    { 
      m_nodes.push_back(fi->fileName());
    }
    
    ++it;
  }
}

// --------------------------------------------------------------------------------
void openMosixWrapper::getNodesData(void)
{
  m_nodesData.clear();
  
  for(QValueList<QString>::const_iterator i=m_nodes.begin();i!=m_nodes.end();i++)
  { 
    QString   dir = QString("/proc/hpc/nodes/") + *i + "/";
    NODEINFO info;
    
    info.m_id = INT(*i);

    // N CPUS
    {
      QFile In(dir + "cpus");
    
      if(In.open(IO_ReadOnly))
      {
        QTextStream stream(&In);

        QString str;

        stream >> str;
      
        info.m_cpus = INT(str);
        In.close();
      }
    }

    // LOAD
    {
      QFile In(dir + "load");
    
      if(In.open(IO_ReadOnly))
      {
        QTextStream stream(&In);

        QString str;

        stream >> str;
      
        info.m_load = INT(str);
        In.close();
      }
    }

    // MEM
    {
      QFile In(dir + "mem");
    
      if(In.open(IO_ReadOnly))
      {
        QTextStream stream(&In);

        QString str;

        stream >> str;
      
        info.m_mem = INT(str);
        In.close();
      }
    }

    // TMEM
    {
      QFile In(dir + "tmem");
    
      if(In.open(IO_ReadOnly))
      {
        QTextStream stream(&In);

        QString str;

        stream >> str;
      
        info.m_tmem = INT(str);
        In.close();
      }
    }

    // SPEED
    {
      QFile In(dir + "speed");
    
      if(In.open(IO_ReadOnly))
      {
        QTextStream stream(&In);

        QString str;

        stream >> str;
      
        info.m_speed = INT(str);
        In.close();
      }
    }

    // STATUS
    {
      QFile In(dir + "status");
    
      if(In.open(IO_ReadOnly))
      {
        QTextStream stream(&In);

        QString str;

        stream >> str;
      
        info.m_status = INT(str);
        In.close();
      }
    }

    m_nodesData.push_back(info);
  }
}

// --------------------------------------------------------------------------------
int openMosixWrapper::isOk(void) const
{
  return(m_isOk);
}

// --------------------------------------------------------------------------------
int openMosixWrapper::where(int pid)
{
  QFile In("/proc/" + STRING(pid) + "/where");
  
  if(!In.open(IO_ReadOnly)) return(-1);

  QTextStream stream(&In);
  
  QString pidStr;
  
  stream >> pidStr;
  
  return(INT(pidStr));
}

