#include "image.h"
#include "task.h"


QValueList<TASK *> TASK::M_TASK_LIST;
int                TASK::M_TASK_NUMBER=1;


// --------------------------------------------------------------------------------
TASK::TASK(QString &error,
           const QString &cmdString,
           const QString &inputFile,
           const QString &outputFile,
           const QString &outputType,
           const QString &attrString,
           const QString &outputDir,
           int w,int h,int jobs) :
  m_cmdString(cmdString),
  m_attrString(attrString),
  m_outputDir(outputDir),
  m_inputFile(inputFile),
  m_outputFile(outputFile),
  m_outputType(outputType),
  m_w(w),m_h(h),m_jobs(jobs),m_status(QUEUED),m_item(0)
{
  // Get the task number
  m_task = M_TASK_NUMBER;
  M_TASK_NUMBER ++;

  // Gap
  int gap = h/jobs;

  if(!isAnimation())
  {
    for (int i=0;i<jobs;i++)
    {
      int start = gap*i;
      int end   = start + (gap-1);

      if(end>m_h-1) end = m_h-1;
    
      QString err;
      JOB *j = new JOB(err,m_task,i+1,cmdString,inputFile,attrString,outputDir,w,h,start,end);

      if(!err.isEmpty())
      {
        error = err;
        return;
      }

      m_jobsList.push_back(j);
    }
  } else {
            CERR("Animation found !\n");
            CERR((QString("S:")+STRING(getFirstFrame()) + "\n"));
            CERR((QString("E:")+STRING(getLastFrame())  + "\n"));
            
            
            int cont = 1;
            for(int i=getFirstFrame();i<=getLastFrame();i++,cont ++)
            {
              QString err;
              
              QString tmp(" ");
              QString tmp1;
              
              tmp1.setNum(i);
              
              tmp += QString("+KFI") + STRING(getFirstFrame());
              tmp += QString(" ");
              tmp += QString("+KFF") + STRING(getLastFrame());
              tmp += QString(" ");              
              tmp += QString("+SF") + tmp1;
              tmp += QString(" ");
              tmp += QString("+EF") + tmp1;
              
              JOB *j = new JOB(err,
                                m_task,
                                cont,
                                cmdString,
                                inputFile,
                                attrString + tmp,
                                outputDir,
                                w,
                                h,
                                0,
                                h-1);

              j->setFrameNumber(i,getLastFrame());
              
              if(!err.isEmpty())
              {
                error = err;
                return;
              }

              m_jobsList.push_back(j);
            }
          }
}

// --------------------------------------------------------------------------------
TASK::~TASK(void)
{
  for(QValueList<JOB *>::iterator i=m_jobsList.begin();i!=m_jobsList.end();i++)
  {
    delete (*i);
  }

  m_jobsList.clear();
}

// --------------------------------------------------------------------------------
bool TASK::recreateJobs(QString &error)
{
  for(QValueList<JOB *>::iterator i=m_jobsList.begin();i!=m_jobsList.end();i++)
  {
    delete (*i);
  }

  m_jobsList.clear();

  // Reset the status
  m_status = QUEUED;
  
  // Gap
  int gap = m_h/m_jobs;

  if(!isAnimation())
  {
    for (int i=0;i<m_jobs;i++)
    {
      int start = gap*i;
      int end   = start + (gap-1);
    
      if(end>m_h-1) end = m_h-1;

      QString err;
      JOB *j = new JOB(err,m_task,i+1,m_cmdString,m_inputFile,m_attrString,m_outputDir,m_w,m_h,start,end);

      if(!err.isEmpty())
      {
        error = err;
        return(false);
      }

      m_jobsList.push_back(j);
    }
  } else {
            CERR("Animation found !\n");
            CERR((QString("S:")+STRING(getFirstFrame()) + "\n"));
            CERR((QString("E:")+STRING(getLastFrame())  + "\n"));
            
            
            int cont = 1;
            for(int i=getFirstFrame();i<=getLastFrame();i++,cont ++)
            {
              QString err;
              
              QString tmp(" ");
              QString tmp1;
              
              tmp1.setNum(i);

              tmp += QString("+KFI") + STRING(getFirstFrame());
              tmp += QString(" ");
              tmp += QString("+KFF") + STRING(getLastFrame());
              tmp += QString(" ");              
              tmp += QString("+SF") + tmp1;
              tmp += QString(" ");
              tmp += QString("+EF") + tmp1;
              
              JOB *j = new JOB(err,
                                m_task,
                                cont,
                                m_cmdString,
                                m_inputFile,
                                m_attrString + tmp,
                                m_outputDir,
                                m_w,
                                m_h,
                                0,
                                m_h-1);

              j->setFrameNumber(i,getLastFrame());
              
              if(!err.isEmpty())
              {
                error = err;
                return(false);
              }

              m_jobsList.push_back(j);
            }
          }
  
  return(true);
}

// --------------------------------------------------------------------------------
bool TASK::run(void)
{
  bool ret = true;

  m_status = RUNNING;
  
  updateItem();
  
  for(QValueList<JOB *>::iterator i=m_jobsList.begin();i!=m_jobsList.end();i++)
  {
    (*i)->setSelectionFlag(true);
    ret &= (*i)->run();
  }

  return(ret);
}

// --------------------------------------------------------------------------------
bool TASK::runAnimation(int s,int e)
{
  bool ret = true;
  
  m_status = RUNNING;
  
  updateItem();
  
  for(QValueList<JOB *>::iterator i=m_jobsList.begin();i!=m_jobsList.end();i++)
  {
    if((*i)->getJobNumber()>=s && (*i)->getJobNumber()<=e)
    {
      (*i)->setSelectionFlag(true);
      ret &= (*i)->run();
    } else (*i)->setSelectionFlag(false);
  }

  return(ret);
}

// --------------------------------------------------------------------------------
void TASK::validateTASK(void)
{
  M_TASK_LIST.push_back(this);
}

// --------------------------------------------------------------------------------
TASK * TASK::createTask(QString &error,
                        const QString &cmdString,
                        const QString &inputFile,
                        const QString &outputFile,
                        const QString &outputType,
                        const QString &attrString,
                        const QString &outputDir,
                        int w,int h,int jobs)
{
  if(cmdString.isEmpty())  {error = "Invalid command ! (Please check the configuration)"; return(0);}
  if(outputDir.isEmpty())  {error = "Invalid output directory ! (Please check the configuration)"; return(0);}
  if(w<=0)               {error = "Invalid width ! (1 <= w <= n)"; return(0);}
  if(h<=0)               {error = "Invalid height ! (1 <= h <= n)"; return(0);}
  if(jobs<=0)            {error = "Invalid jobs number ! (1 <= Jobs <= n)"; return(0);}

  TASK *ret = new TASK(error,cmdString,inputFile,outputFile,outputType,attrString,outputDir,w,h,jobs);

  return(ret);
}

// --------------------------------------------------------------------------------
TASK * TASK::findTask(int num)
{
  for(QValueList<TASK *>::iterator i=M_TASK_LIST.begin();i!=M_TASK_LIST.end();i++)
  {
    if((*i)->m_task==num) return(*i);
  }

  return(0);
}

// --------------------------------------------------------------------------------
void TASK::resetAllItems(void)
{
  for(QValueList<JOB *>::iterator i=m_jobsList.begin();i!=m_jobsList.end();i++)
  {
     (*i)->setItem(0);
  }
}

// --------------------------------------------------------------------------------
const QString TASK::getTaskStatusStr(void)  const
{
  switch(m_status)
  {
    case QUEUED:
      return("QUEUED");

    case RUNNING:
      return("RUNNING");

    case FINISHED:
      return("FINISHED");

    case PAUSED:
      return("PAUSED");
            
    default:
    return("UNKNOWN");
  }
}

// --------------------------------------------------------------------------------
JOB * TASK::findJob(int number)
{
  for(QValueList<JOB *>::iterator i=m_jobsList.begin();i!=m_jobsList.end();i++)
  {
    if((*i)->getJobNumber() == number) return(*i);
  }
  
  return(0);
}

 // --------------------------------------------------------------------------------
void TASK::jobFinishCallback(void)
{
  bool finishedJob = true;
  
  for(QValueList<JOB *>::iterator i=m_jobsList.begin();i!=m_jobsList.end();i++)
  {
    if((*i)->getSelectionFlag() && (*i)->getJobStatusStr() != "FINISHED") finishedJob=false;
  }
  
  if(finishedJob)
  {
    m_status = FINISHED;
    
    updateItem();
    
    // Update listView
    if(m_item->isSelected())
    {
      m_item->listView()->clearSelection();
      m_item->listView()->setSelected(m_item,true);
    }

    if(!isAnimation())
    {   
      // Save image
      imageBuilder ib;
      bool         ret;
   
      ret = ib.buildImage(m_w,m_h,m_jobsList);
    
      if(ret)
      { 
        ib.saveImage(m_outputFile,QString(m_outputType).upper());
      }
    } else {
              for(QValueList<JOB *>::iterator i=m_jobsList.begin();i!=m_jobsList.end();i++)
              {
                if((*i)->getSelectionFlag())
                { 
                  // Save image
                  imageBuilder ib;
                  bool         ret;

                  ret = ib.buildImage(m_w,m_h,*i);
              
                  if(ret)
                  { 
                    QString str;
                    QString prefix,postfix;
                    
                    int pos = m_outputFile.findRev(".");
                    if(pos>0) prefix = m_outputFile.mid(0,pos);
                         else prefix = m_outputFile;

                    if(pos>0) postfix = m_outputFile.mid(pos+1);
                         else postfix = "";
                         
                    CERR((QString("Pr : ") + prefix + "\n"));
                    CERR((QString("Po : ") + postfix + "\n"));
                    
                    int field = strlen(STRING((*i)->getFrameTotal()));
  
                    QString val;
                    val.setNum((*i)->getFrameNumber());
  
                    while((int) val.length()!=field)
                    {
                      val = "0" + val;
                    }
                    
                    str  = prefix;
                    str += val;
                    str += ".";
                    str += postfix;

                    CERR((QString("Str : ") + str + "\n"));
                                                                 
                    ib.saveImage(str,QString(m_outputType).upper());
                  }
                }
              }
            } 
  }
  
}

// --------------------------------------------------------------------------------
void TASK::updateItem(void)
{
  if(!m_item) return;

  if(m_status==FINISHED)
  { 
    int exitStatus=0;
    
    for(QValueList<JOB *>::iterator i=m_jobsList.begin();i!=m_jobsList.end();i++)
    {
      exitStatus|=(*i)->getExitStatus();  
    }

    if(exitStatus)
    {
      QImage img = QImage::fromMimeSource("error.png");
      m_item->setPixmap(0,img);
    } else m_item->setPixmap(0,QImage());
  }
  
  QString size = STRING(getWidth()) + "x" + STRING(getHeight());

  m_item->setText(0,STRING(m_task)); 
  m_item->setText(1,STRING(m_jobs));
  m_item->setText(2,size);
  m_item->setText(3,m_inputFile);
  m_item->setText(4,m_outputFile);
  m_item->setText(5,getTaskStatusStr());
}

// --------------------------------------------------------------------------------
void TASK::deleteAllTasks(void)
{
  for(QValueList<TASK *>::iterator i=M_TASK_LIST.begin();i!=M_TASK_LIST.end();i++)
  {
    delete(*i);    
  }
  
  M_TASK_LIST.clear();
}

// --------------------------------------------------------------------------------
bool TASK::deleteTask(TASK *task)
{
  for(QValueList<TASK *>::iterator i=M_TASK_LIST.begin();i!=M_TASK_LIST.end();i++)
  {
    if((*i)==task)    
    {
      M_TASK_LIST.remove(*i);
      delete task;
      return(true);
    }
  }
  
  return(false);
}

// --------------------------------------------------------------------------------
void TASK::stopTask(void)
{
  for(QValueList<JOB *>::iterator i=m_jobsList.begin();i!=m_jobsList.end();i++)
  {
    (*i)->stop();
  }  
}

// --------------------------------------------------------------------------------
void TASK::pauseTask(void)
{
  for(QValueList<JOB *>::iterator i=m_jobsList.begin();i!=m_jobsList.end();i++)
  {
    if((*i)->getJobStatusStr()=="RUNNING")      (*i)->pause_job();
  }
  
  m_status = PAUSED; 
  
  updateItem(); 
}

// --------------------------------------------------------------------------------
void TASK::continueTask(void)
{
  for(QValueList<JOB *>::iterator i=m_jobsList.begin();i!=m_jobsList.end();i++)
  {
    if((*i)->getJobStatusStr()=="PAUSED") (*i)->continue_job();
  }  
  
  m_status = RUNNING;  
  
  updateItem();
}

// --------------------------------------------------------------------------------
bool TASK::isAnimation() const
{
  return((m_attrString.contains("+KFI") || m_attrString.contains("+KFF")) ? true : false);
}

// --------------------------------------------------------------------------------
int TASK::getFirstFrame(void) const
{
  int ret = 0;
  int pos = m_attrString.findRev("+KFI");
  
  if(pos==-1) return(0);
  
  const char *p = (const char *) m_attrString;
  
  sscanf(p+pos+4,"%d",&ret);
 
  return(ret);
}

// --------------------------------------------------------------------------------
int TASK::getLastFrame(void) const
{
  int ret = 0;
  int pos = m_attrString.findRev("+KFF");
  
  if(pos==-1) return(0);
  
  const char *p = (const char *) m_attrString;
  
  sscanf(p+pos+4,"%d",&ret);
 
  return(ret);
}

// --------------------------------------------------------------------------------
JOB *  TASK::getJobFrame(int f)
{
  for(QValueList<JOB *>::iterator i=m_jobsList.begin();i!=m_jobsList.end();i++)
  {
    if((*i)->getFrameNumber() == f) return(*i);
  }
  
  return(0);
}
