#ifndef _TASK_H
#define _TASK_H

#include <stdlib.h>
#include <qstring.h>
#include <qptrlist.h>
#include <qlistview.h>

#include "utility.h"
#include "job.h"


class TASK
{
  private:

  static QValueList<TASK *> M_TASK_LIST;
  static int                M_TASK_NUMBER;

  QString        m_cmdString;
  QString        m_attrString;
  QString        m_outputDir;
  QString        m_inputFile;
  QString        m_outputFile;
  QString        m_outputType;
  int            m_w,m_h;
  int            m_task;
  int            m_jobs;
  enum           {QUEUED,RUNNING,FINISHED,PAUSED} m_status;
  QListViewItem *m_item;

  QValueList<JOB *> m_jobsList;

  
  TASK(QString &error,
       const QString &cmdString,
       const QString &inputFile,
       const QString &outputFile,
       const QString &outputType,
       const QString &attrString,
       const QString &outputDir,
       int w,int h,int jobs);

  public:

  ~TASK(void);
  
  bool          run(void);
  bool          runAnimation(int s,int e);
  void          validateTASK(void);
  static TASK * findTask(int num);
  static TASK * createTask(QString &error,
                           const QString &cmdString,
                           const QString &inputFile,
                           const QString &outputFile,
                           const QString &outputType,
                           const QString &attrString,
                           const QString &outputDir,
                           int w,int h,int jobs);
  JOB *         findJob(int number);
  void          jobFinishCallback(void);
  bool          recreateJobs(QString &error);
  void          updateItem(void);
  static void   deleteAllTasks(void);
  static bool   deleteTask(TASK *task);
  void          stopTask(void);
  void          pauseTask(void);
  void          continueTask(void);
    
  // --------------------------------------------------------------------------------
  // GET
  // --------------------------------------------------------------------------------

  int                              getJobs(void)           const {return(m_jobs);}
  int                              getTaskN(void)          const {return(m_task);}
  const QString &                  getInputFile(void)      const {return(m_inputFile);}
  const QString &                  getOutputFile(void)     const {return(m_outputFile);}
  const QString &                  getOutputType(void)     const {return(m_outputType);}
  const QString &                  getAttributeList(void)  const {return(m_attrString);}  
  int                              getTaskStatus(void)     const {return(m_status);}  
  int                              getWidth(void)          const {return(m_w);}
  int                              getHeight(void)         const {return(m_h);}  
  QListViewItem *                  getItem(void)           const {return(m_item);}
  static const QValueList<TASK *>  getTaskList(void)             {return(M_TASK_LIST);}
  QValueList<JOB *> &              getJobList(void)              {return(m_jobsList);}
  const QString                    getTaskStatusStr(void)  const;  
  
  bool                             isAnimation() const;
  int                              getFirstFrame(void) const;
  int                              getLastFrame(void) const;
  JOB *                            getJobFrame(int f);
  
  
  // --------------------------------------------------------------------------------
  // SET
  // --------------------------------------------------------------------------------

  void setInputFile(const QString &inputFile)    {m_inputFile = inputFile;}
  void setOutputFile(const QString &outputFile)  {m_outputFile = outputFile;}
  void setAttributeList(const QString &attrList) {m_attrString = attrList;}
  void setJobs(int jobs)                         {m_jobs=jobs;}
  void setWidth(int w)                           {m_w = w;}
  void setHeight(int h)                          {m_h = h;}
  void setItem(QListViewItem *i)                 {m_item = i;}
  void setOutputType(const QString &outputType)  {m_outputType = outputType;}
  void resetAllItems(void);
};


#endif

