#include "task.h"
#include "utility.h"
#include "configuration.h"
#include "xml.h"

#include <qmessagebox.h>


#define DOC_POVMOSIX "POVmosix_1_0"


// --------------------------------------------------------------------------------
const QString getElement(QDomElement &e,const QString &name)
{
   QDomNodeList nl = e.elementsByTagName(name);
   if(nl.length()==1)
   {
     QDomNode n = nl.item(0);
     if(n.isNull() || (!n.isElement()))
     {
       CERR((QString("Wrong node : ") + name + "\n"));
     } else return((const char *) n.toElement().text());
   }
   
  return("");   
}

// --------------------------------------------------------------------------------
void saveTASKS(QWidget *d,const QString &filename)
{
const QValueList<TASK *> taskList = TASK::getTaskList();

  QFile out(filename);
  
  // Delete old file
  out.remove();

  if(!out.open(IO_ReadWrite))
  {
    QMessageBox::critical(d, 
                          "Error",
                          "Cannot open output file.",
                          QMessageBox::Ok | QMessageBox::Default,
                          QMessageBox::NoButton);
  }
  

  QDomDocument doc(DOC_POVMOSIX);
  QDomElement root = doc.createElement("root");
  doc.appendChild(root);
  
  for(QValueList<TASK *>::const_iterator i=taskList.begin();i!=taskList.end();i++)
  {
    QDomElement task = doc.createElement("task");
    root.appendChild(task);

    QDomElement tag = doc.createElement("input");
    task.appendChild(tag);
    tag.appendChild(doc.createTextNode((*i)->getInputFile()));

    tag = doc.createElement("output");
    task.appendChild(tag);
    tag.appendChild(doc.createTextNode((*i)->getOutputFile()));

    tag = doc.createElement("outputType");
    task.appendChild(tag);
    tag.appendChild(doc.createTextNode((*i)->getOutputType()));

    tag = doc.createElement("attributes");
    task.appendChild(tag);
    tag.appendChild(doc.createTextNode((*i)->getAttributeList()));

    tag = doc.createElement("jobs");
    task.appendChild(tag);
    tag.appendChild(doc.createTextNode(STRING((*i)->getJobs())));

    tag = doc.createElement("w");
    task.appendChild(tag);
    tag.appendChild(doc.createTextNode(STRING((*i)->getWidth())));

    tag = doc.createElement("h");
    task.appendChild(tag);
    tag.appendChild(doc.createTextNode(STRING((*i)->getHeight())));
  }
  
  QString xml = doc.toString();  
  QTextStream stream(&out);

  stream << xml;
}

// --------------------------------------------------------------------------------
void loadTASKS(QWidget *d,QListView *l,const QString &filename)
{
  QDomDocument doc(DOC_POVMOSIX);
  
  QFile file(filename);
  if (!file.open(IO_ReadOnly))
  {
    QMessageBox::critical(d, 
                          "Error",
                          "Cannot open input file.",
                          QMessageBox::Ok | QMessageBox::Default,
                          QMessageBox::NoButton);

    return;
  }
  
  if (!doc.setContent(&file))
  {
    QMessageBox::critical(d, 
                          "Error",
                          "Cannot open input file.",
                          QMessageBox::Ok | QMessageBox::Default,
                          QMessageBox::NoButton);
  
    file.close();
    return;
  }
  
  file.close();
  
  if(doc.doctype().name()!=DOC_POVMOSIX)
  {
    QMessageBox::critical(d, 
                          "Error",
                          "Invalid file.",
                          QMessageBox::Ok | QMessageBox::Default,
                          QMessageBox::NoButton);
  
    return;
  }
  
    
  QDomElement docElem = doc.documentElement();
  

  QDomNode n = docElem.firstChild();
  while(!n.isNull())
  {
    QDomElement e = n.toElement(); // try to convert the node to an element.
    if(!e.isNull())
    {
      if(e.tagName()=="task")
      { 
        QString inputFile(getElement(e,"input"));
        QString outputFile(getElement(e,"output"));
        QString outputType(getElement(e,"outputType"));
        QString attributes(getElement(e,"attributes"));
        QString jobs(getElement(e,"jobs"));
        QString w(getElement(e,"w"));        
        QString h(getElement(e,"h"));                

        if((!inputFile.isEmpty())  && 
           (!outputFile.isEmpty()) && 
           (!outputType.isEmpty()) && 
           (!jobs.isEmpty())       && 
           (!w.isEmpty())          && 
           (!h.isEmpty()))
         {
           CFG     cfg;
           QString error;
           
           TASK *t = TASK::createTask(error,
                                      cfg.getPovrayPath(),
                                      inputFile,
                                      outputFile,
                                      outputType,
                                      attributes,
                                      cfg.getTmpDir(),
                                      w.toInt(),h.toInt(),jobs.toInt());
                                      
          if(!error.isEmpty())
          {
            QMessageBox::warning(d, 
                                 "TASK creation error !",
                                  error,
                                  QMessageBox::Ok | QMessageBox::Default,
                                  QMessageBox::NoButton);
      
          } else {
                    t->validateTASK();
                    
                    QString sJobs   = STRING<int>(t->getJobs());
                    QString sTaskN  = STRING<int>(t->getTaskN());
                    QString sStatus = t->getTaskStatusStr();
                    QString size    = w + "x" + h;
    
                    QListViewItem *i = new QListViewItem(l,
                                                          sTaskN,
                                                          sJobs,
                                                          size,
                                                          t->getInputFile(),
                                                          t->getOutputFile(),
                                                          sStatus);
    
                    t->setItem(i);
                  }
                                                
        }
      }
    }
    n = n.nextSibling();
  }

}
